/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_H_
#define DOCA_STA_H_

#include <stdint.h>

#include <doca_compat.h>
#include <doca_error.h>
#include <doca_dpa.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_devinfo;
struct doca_dev;
struct doca_pe;
struct doca_sta;

/**
 * @brief Get whether the DOCA device supports storage target acceleration ('STA')
 *
 * @param[in] devinfo - the device to query
 *
 * @return
 * DOCA_SUCCESS - in case of the DOCA device queried has STA support
 * DOCA_ERROR_INVALID_VALUE - received invalid input
 * DOCA_ERROR_NOT_SUPPORTED - the device queried does not support STA
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_cap_is_supported(const struct doca_devinfo *devinfo);

/**
 * @brief Create a DOCA STA Context
 *
 * This function creates a DOCA STA context given a DOCA device.
 * The context represents a program on the STA that is referenced
 * by the host process that called the context creation API
 *
 * @param[in] dev - DOCA device
 * @param[out] sta - created context
 *
 * @return
 * DOCA_SUCCESS - in case of success
 * DOCA_ERROR_INVALID_VALUE - received invalid input
 * DOCA_ERROR_DRIVER - in case of error in a DOCA driver call
 * DOCA_ERROR_NOT_SUPPORTED - the device does not support STA
 * DOCA_ERROR_NO_MEMORY - in case of failure in internal memory allocation
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_create(struct doca_dev *dev, struct doca_sta **sta);

/**
 * @brief Destroy a DOCA STA context
 *
 * This function destroys STA context created by doca_sta_create()
 *
 * @param[in] sta - Previously created STA context
 *
 * @return
 * DOCA_SUCCESS - in case of success
 * DOCA_ERROR_INVALID_VALUE - received invalid input
 * DOCA_ERROR_DRIVER - in case of error in a DOCA driver call
 * DOCA_ERROR_IN_USE - in case the STA context is still used by another DOCA context
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_destroy(struct doca_sta *sta);

/**
 * @brief Support STA functionality on a given device.
 *
 * @param [in] sta - Previously created STA context
 * @param [in] dev - DOCA Dev instance with appropriate capability
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * doca_error code - in case of failure:
 * DOCA_ERROR_INVALID_VALUE - if an invalid input had been received
 * DOCA_ERROR_NO_MEMORY - if reached to number of devices reported by doca_sta_get_max_devs()
 * DOCA_ERROR_BAD_STATE - if sta in wrong  state
 * DOCA_ERROR_ALREADY_EXIST - if doca_dev already exists in doca_mmap
 * DOCA_ERROR_NOT_SUPPORTED - provided dev does not support the RC transport type
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_add_dev(struct doca_sta *sta, struct doca_dev *dev);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_set_max_sta_io(struct doca_sta *sta, uint16_t max_io_num);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_sta_io(const struct doca_sta *sta, uint16_t *max_io_num);

/**
 * @brief Convert doca_sta instance into a generalized context for use with doca core objects.
 *
 * @param [in] sta - Pointer to doca_sta instance.
 *
 * @return
 * Non NULL upon success, NULL otherwise.
 */
DOCA_EXPERIMENTAL
struct doca_ctx *doca_sta_as_ctx(struct doca_sta *sta);

/**
 * @brief Get the max number of the supported devices by the STA engine
 *
 * @param[in] max_devs - max devices
 *
 * @return
 * - DOCA_SUCCESS - in case of success
 * - DOCA_ERROR_INVALID_VALUE - received invalid input
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_devs(uint32_t *max_devs);

/**
 * @brief Retrieve the total number of EUs available to the application
 *
 * @param[out] max_eus - number of total available EUs
 *
 * @return
 * - DOCA_SUCCESS - in case of success
 * - DOCA_ERROR_INVALID_VALUE - received invalid input
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_num_eus_available(const struct doca_sta *sta, uint32_t *max_eus);

/**
 * @brief Retrieve the maximum number of connected QPs per EU
 *
 * @param[out] max_connected_qps - maximum number of connected QPs
 *
 * @return
 * - DOCA_SUCCESS - in case of success
 * - DOCA_ERROR_INVALID_VALUE - received invalid input
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_num_connected_qp_per_eu(const struct doca_sta *sta, uint32_t *max_connected_qps);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_subsys(uint32_t *max_subsys);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_ns_per_subs(uint32_t *max_ns_per_subsys);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_qps(uint32_t *max_qps);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_io_threads(uint32_t *max_io_threads);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_io_size(uint32_t *max_io_size);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_io_num_per_dev(const struct doca_sta *sta, uint32_t *max_ios);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_io_queue_size(const struct doca_sta *sta, uint32_t *max_io_queue_size);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_min_ioccsz(uint32_t *ioccsz);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_ioccsz(uint32_t *ioccsz);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_min_iorcsz(uint32_t *iorcsz);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_iorcsz(uint32_t *iorcsz);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_icdoff(uint32_t *icdoff);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_be(uint32_t *max_be);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_get_max_qs_per_be(uint32_t *max_qs_per_be);

DOCA_EXPERIMENTAL
bool doca_sta_is_logical_block_size_supported(uint32_t size);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_H_ */
