/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_EVENT_H_
#define DOCA_STA_EVENT_H_

#include <stdint.h>
#include <stdbool.h>

#include <doca_error.h>
#include <doca_types.h>

#include <doca_sta_handle.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_sta;
struct doca_sta_io;
struct dev_sta_event;

struct doca_sta_event_transport_err;

/**
 * @brief Function to execute on STA event notification.
 *
 * @details This function is called by doca_pe_progress() when related task received an event notification from DPA.
 *
 * NOTE: the RDMA error events will be notified in the context of the doca_sta_io thread(s).
 * The same doca_sta_io thread that initiated connect for the QP.
 * The time-out events will be notified in the context of the doca_sta thread.
 *
 * @param [in] event
 * STA event.
 * @param [in] user_data
 * The user data parameter specified during call to doca_sta_event_register_cb API.
 */
typedef void (*doca_sta_event_transport_err_cb_t)(const struct doca_sta_event_transport_err *event,
						  union doca_data user_data);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_transport_err_get_qp_handle(const struct doca_sta_event_transport_err *event,
							const struct doca_sta_qp_handle **qp_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_transport_err_get_operation(const struct doca_sta_event_transport_err *event,
							uint32_t *operation);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_transport_err_get_syndrome(const struct doca_sta_event_transport_err *event,
						       uint32_t *syndrome);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_transport_err_get_vendor_syndrome(const struct doca_sta_event_transport_err *event,
							      uint32_t *vendor_syndrome);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_event_transport_err_register_cb(struct doca_sta_io *sta_io,
							 doca_sta_event_transport_err_cb_t event_cb,
							 union doca_data user_data);

struct doca_sta_event_be_timeout;

typedef void (*doca_sta_event_be_timeout_cb_t)(const struct doca_sta_event_be_timeout *event,
					       union doca_data user_data);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_be_timeout_register_cb(struct doca_sta *sta,
						   doca_sta_event_be_timeout_cb_t event_cb,
						   union doca_data user_data);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_be_timeout_get_be_handle(const struct doca_sta_event_be_timeout *event,
						     const struct doca_sta_be_handle **be_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_be_timeout_get_be_queue_handle(const struct doca_sta_event_be_timeout *event,
							   const struct doca_sta_be_q_handle **be_queue_handle);

struct doca_sta_event_eu_err;

typedef void (*doca_sta_event_eu_err_cb_t)(const struct doca_sta_event_eu_err *event, union doca_data user_data);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_eu_err_register_cb(struct doca_sta *sta,
					       doca_sta_event_eu_err_cb_t event_cb,
					       union doca_data user_data);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_eu_err_is_fatal_error(const struct doca_sta_event_eu_err *event, bool *is_fatal_error);

/* Relevant for non-fatal errors only */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_event_eu_err_get_eu_handle(const struct doca_sta_event_eu_err *event,
						 const struct doca_sta_eu_handle **eu_handle);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_EVENT_H_ */
