/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_IO_H_
#define DOCA_STA_IO_H_

#include <stdint.h>

#include <doca_compat.h>
#include <doca_error.h>
#include <doca_types.h>

#include <doca_sta_handle.h>
#include <doca_sta_task.h>

#ifdef __cplusplus
extern "C" {
#endif

struct doca_pe;
struct doca_dev;

struct doca_sta;
struct doca_sta_io;

/**
 * @brief Create a DOCA STA IO Context
 *
 * This function creates a DOCA STA IO context attached to a DOCA STA context (control context).
 * The IO context allows the user to perform IO transactions using the offload engine.
 *
 * @param[in] sta - Previously created STA context
 * @param[out] sta_io - created IO context
 *
 * @return
 * DOCA_SUCCESS - in case of success
 * Error code - in case of failure. see doca_error_t.
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_create(struct doca_sta *sta, struct doca_sta_io **sta_io);

/**
 * @brief Destroy a DOCA STA IO context
 *
 * This function destroys STA IO context created by doca_sta_io_create()
 *
 * @param[in] sta_io - Previously created STA IO context
 *
 * @return
 * DOCA_SUCCESS - in case of success
 * Error code - in case of failure. see doca_error_t.
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_destroy(struct doca_sta_io *sta_io);

/**
 * @brief Convert doca_sta_io instance into a generalized context for use with doca core objects.
 *
 * @param [in] sta_io - Pointer to doca_sta_io instance.
 *
 * @return
 * Non NULL upon success, NULL otherwise.
 */
DOCA_EXPERIMENTAL
struct doca_ctx *doca_sta_io_as_ctx(struct doca_sta_io *sta_io);

/**
 * @brief This method sets the STA IO disconnect tasks configuration
 *
 * @param [in] sta_io
 * The STA IO context to config
 * @param [in] task_completion_cb
 * Task completion callback
 * @param [in] task_error_cb
 * Task error callback
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * Error code - in case of failure. see doca_error_t.
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_task_disconnect_set_conf(struct doca_sta_io *sta_io,
						  doca_sta_task_completion_cb_t task_completion_cb,
						  doca_sta_task_completion_cb_t task_error_cb);

/**
 * @brief This method allocates and initializes a STA IO disconnect task
 *
 * @param [in] sta_io
 * The STA IO context to config
 * @param [in] user_data
 * doca_data to attach to the task
 * @param [in] qp_handle
 * The handle of the QP that should be disconnected
 * @param [out] task
 * memcpy task to allocate
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * doca_error code - in case of failure:
 * - DOCA_ERROR_NO_MEMORY - No more tasks to allocate
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_task_disconnect_alloc_init(struct doca_sta_io *sta_io,
						    union doca_data user_data,
						    struct doca_sta_qp_handle *qp_handle,
						    struct doca_sta_producer_task_send **task);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_IO_H_ */
