/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_IO_NON_OFFLOAD_H_
#define DOCA_STA_IO_NON_OFFLOAD_H_

#include <stdint.h>
#include <stdbool.h>

#include <doca_error.h>
#include <doca_types.h>

#include <doca_sta_handle.h>
#include <doca_sta_task.h>

#ifdef __cplusplus
extern "C" {
#endif

#define DOCA_STA_NVMEF_COMPLETION_SIZE (16)

typedef uint8_t doca_sta_nvmef_completion_t[DOCA_STA_NVMEF_COMPLETION_SIZE];

struct doca_sta_io;

/**
 * @brief Function to execute on new STA IO non-offload command notification.
 *
 * @details This function is called by doca_pe_progress() when related task received a new non-offload command from DPA.
 *
 * @param [in] qp_handle
 * STA QP.
 * @param [in] nvme_cmd
 * NVME command.
 * @param [in] payload
 * The payload data (if any) belonged to the nvme command.
 * @param [in] payload_len
 * The payload length of the payload. The maximum length of the payload buffer can be obtained
 * by calling to doca_sta_get_max_io_size API.
 * @param [in] payload_valid
 * The hint to denote if the nvme command has been retrieved from initiator and stored in the payload buffer.
 * @param [in] non_offload_user_data
 * The user data attached to the nvme command.
 */
typedef void (*doca_sta_io_non_offload_cb_t)(struct doca_sta_qp_handle *qp_handle,
					     union doca_data user_data,
					     const uint8_t *nvme_cmd,
					     uint8_t *payload,
					     uint32_t payload_len,
					     bool payload_valid,
					     union doca_data non_offload_user_data);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_non_offload_register_cb(struct doca_sta_io *sta_io,
						 doca_sta_io_non_offload_cb_t non_offload_cb,
						 union doca_data user_data);

/**
 * @brief This method sets the STA IO non offload RDMA WRITE tasks configuration
 *
 * @param [in] sta_io
 * The STA IO context to config
 * @param [in] task_completion_cb
 * Task completion callback
 * @param [in] task_error_cb
 * Task error callback
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * Error code - in case of failure. see doca_error_t.
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_task_non_offload_set_rdma_write_send_conf(struct doca_sta_io *sta_io,
								   doca_sta_task_completion_cb_t task_completion_cb,
								   doca_sta_task_completion_cb_t task_error_cb);
/**
 * @brief This method sets the STA IO non offload RDMA READ tasks configuration
 *
 * @param [in] sta_io
 * The STA IO context to config
 * @param [in] task_completion_cb
 * Task completion callback
 * @param [in] task_error_cb
 * Task error callback
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * Error code - in case of failure. see doca_error_t.
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_task_non_offload_set_rdma_read_conf(struct doca_sta_io *sta_io,
							     doca_sta_task_completion_cb_t task_completion_cb,
							     doca_sta_task_completion_cb_t task_error_cb);

/**
 * @brief This method allocates and initializes a STA IO RDMA WRITE with RDMA SEND task
 *
 * @param [in] sta_io
 * The STA IO context to config.
 * @param [in] user_data
 * doca_data to attach to the task.
 * @param [in] qp_handle
 * The handle of the QP that should be disconnected.
 * @param [in] non_offload_user_data
 * doca_data attached to the original nvme command.
 * @param [out] task
 * memcpy task to allocate
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * doca_error code - in case of failure:
 * - DOCA_ERROR_NO_MEMORY - No more tasks to allocate
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_task_non_offload_rdma_write_send_alloc_init(struct doca_sta_io *sta_io,
								     union doca_data user_data,
								     struct doca_sta_qp_handle *qp_handle,
								     doca_sta_nvmef_completion_t completion,
								     union doca_data non_offload_user_data,
								     struct doca_sta_producer_task_send **task);

/**
 * @brief This method allocates and initializes a STA IO RDMA SEND task
 *
 * @param [in] sta_io
 * The STA IO context to config.
 * @param [in] user_data
 * doca_data to attach to the task.
 * @param [in] qp_handle
 * The handle of the QP that should be disconnected.
 * @param [in] non_offload_user_data
 * doca_data attached to the original nvme command.
 * @param [out] task
 * memcpy task to allocate
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * doca_error code - in case of failure:
 * - DOCA_ERROR_NO_MEMORY - No more tasks to allocate
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_task_non_offload_rdma_send_alloc_init(struct doca_sta_io *sta_io,
							       union doca_data user_data,
							       struct doca_sta_qp_handle *qp_handle,
							       doca_sta_nvmef_completion_t completion,
							       union doca_data non_offload_user_data,
							       struct doca_sta_producer_task_send **task);

/**
 * @brief This method allocates and initializes a STA IO RDMA READ task
 *
 * @param [in] sta_io
 * The STA IO context to config.
 * @param [in] user_data
 * doca_data to attach to the task.
 * @param [in] qp_handle
 * The handle of the QP that should be disconnected.
 * @param [in] non_offload_user_data
 * doca_data attached to the original nvme command.
 * @param [out] task
 * memcpy task to allocate
 *
 * @return
 * DOCA_SUCCESS - in case of success.
 * doca_error code - in case of failure:
 * - DOCA_ERROR_NO_MEMORY - No more tasks to allocate
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_task_non_offload_rdma_read_alloc_init(struct doca_sta_io *sta_io,
							       union doca_data user_data,
							       struct doca_sta_qp_handle *qp_handle,
							       union doca_data non_offload_user_data,
							       struct doca_sta_producer_task_send **task);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_IO_NON_OFFLOAD_H_ */
