/*
 * Copyright (c) 2024 NVIDIA CORPORATION & AFFILIATES, ALL RIGHTS RESERVED.
 *
 * This software product is a proprietary product of NVIDIA CORPORATION &
 * AFFILIATES (the "Company") and all right, title, and interest in and to the
 * software product, including all associated intellectual property rights, are
 * and shall remain exclusively with the Company.
 *
 * This software product is governed by the End User License Agreement
 * provided with the software product.
 *
 */

#ifndef DOCA_STA_IO_QP_H_
#define DOCA_STA_IO_QP_H_

#include <stdint.h>

#include <doca_compat.h>
#include <doca_error.h>

#include <doca_sta_handle.h>

#ifdef __cplusplus
extern "C" {
#endif

struct rdma_cm_id;
struct doca_dev;
struct doca_sta_io;

/**
 * @brief Allocate an 'empty' STA QP
 * The QP can be used later to accept a remote connection
 * represented by cm_id.
 *
 * @param[in] sta_io - Previously created STA IO context
 * @param[in] dev - The device to attach to the STA QP
 * @param[out] qp_handle - STA QP handle
 *
 * @return
 * DOCA_SUCCESS - in case of success
 * Error code - in case of failure. see doca_error_t.
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_alloc(struct doca_sta_io *sta_io,
				  const struct doca_dev *dev,
				  struct doca_sta_qp_handle **qp_handle);

/**
 * @brief Accept a new remote connection
 *
 * This function accept (connect) a remote QP represented by cm_id to the previously
 * allocated STA QP.
 *
 * @param[in] sta_io - Previously created STA IO context
 * @param[in] qp_handle - previously allocated STA QP
 * @param[in] dev - The device to attach to the STA QP
 * @param[in] cm_id - RDMA CM ID object that carries the remote connection details
 * @param[in] subsys - subsystem details
 *
 * @return
 * DOCA_SUCCESS - in case of success
 * Error code - in case of failure. see doca_error_t.
 */

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_accept(struct doca_sta_io *sta_io,
				   struct doca_sta_qp_handle *qp_handle,
				   const struct doca_dev *dev,
				   struct rdma_cm_id *cm_id,
				   const void *priv_data,
				   uint32_t priv_size,
				   struct doca_sta_subs_handle *subs_handle);

/**
 * @brief Create a newly connected STA QP
 *
 * This function connect a remote QP represented by cm_id to a newly created
 * STA QP, that represents an offloaded QP.
 *
 * @param[in] sta_io - Previously created STA IO context
 * @param[in] dev - The device to attach to the STA QP
 * @param[in] cm_id - RDMA CM ID object that carries the remote connection details
 * @param[in] subsys - subsystem details
 * @param[out] qp_handle - STA QP handle
 *
 * @return
 * DOCA_SUCCESS - in case of success
 * Error code - in case of failure. see doca_error_t.
 */
DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_connect(struct doca_sta_io *sta_io,
				    const struct doca_dev *dev,
				    struct rdma_cm_id *cm_id,
				    const void *priv_data,
				    uint32_t priv_size,
				    struct doca_sta_subs_handle *subs_handle,
				    struct doca_sta_qp_handle **qp_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_connect_set_sq_size(struct doca_sta_io *sta_io,
						struct doca_sta_qp_handle *qp_handle,
						uint16_t sq_size);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_connect_established(struct doca_sta_io *sta_io, struct doca_sta_qp_handle *qp_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_destroy(struct doca_sta_io *sta_io, struct doca_sta_qp_handle *qp_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_add_subsystem(struct doca_sta_io *sta_io,
					  struct doca_sta_subs_handle *subs_handle,
					  struct doca_sta_qp_handle *qp_handle);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_get_id(struct doca_sta_qp_handle *qp_handle, uint32_t *qp_id);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_get_port_id(struct doca_sta_qp_handle *qp_handle, uint16_t *port_id);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_get_group_id(struct doca_sta_qp_handle *qp_handle, uint16_t *group_id);

DOCA_EXPERIMENTAL
doca_error_t doca_sta_io_qp_get_index_in_group(struct doca_sta_qp_handle *qp_handle, uint16_t *qp_idx);

#ifdef __cplusplus
}
#endif

#endif /* DOCA_STA_IO_QP_H_ */
